@@ -1,9 +1,10 @@
 as_1_based_dim <- function(x) {
   x <- as.integer(x)
-  
-  if (any(x == 0))
+
+  if (any(x == 0)) {
     value_error("Dimension is 1-based, but found 0.")
-  
+  }
+
   ifelse(x > 0, x - 1, x)
 }
 
@@ -15,11 +16,12 @@ as_1_based_tensor <- function(x) {
   with_no_grad({
     if (!any(x$shape == 0)) {
       e <- torch_min(torch_abs(x))$to(dtype = torch_int())
-      if (e$item() == 0)
-        runtime_error("Indices/Index start at 1 and got a 0.")  
+      if (e$item() == 0) {
+        runtime_error("Indices/Index start at 1 and got a 0.")
+      }
     }
-    
-    out <- x - (x > 0)$to(dtype = x$dtype)  
+
+    out <- x - (x > 0)$to(dtype = x$dtype)
   })
   out
 }
@@ -41,13 +43,13 @@ do_call <- function(fun, args) {
 }
 
 call_c_function <- function(fun_name, args, expected_types, nd_args, return_types, fun_type) {
-  
   fun_name <- create_fn_name(fun_name, fun_type, nd_args, args, expected_types)
-  f <- getNamespace('torch')[[fun_name]]
-  
-  if (is.null(f))
+  f <- getNamespace("torch")[[fun_name]]
+
+  if (is.null(f)) {
     value_error("{fun_name} does not exist")
-  
+  }
+
   out <- do_call(f, args)
   out
 }
