@@ -1,5 +1,5 @@
 Dataset <- R6::R6Class(
-  classname = "dataset", 
+  classname = "dataset",
   lock_objects = FALSE,
   public = list(
     .getitem = function(index) {
@@ -13,55 +13,62 @@ is_map_dataset <- function(x) {
 }
 
 get_init <- function(x) {
-  
-  if (!is.null(x$public_methods$initialize))
+  if (!is.null(x$public_methods$initialize)) {
     return(x$public_methods$initialize)
-  else
+  } else {
     return(get_init(x$get_inherit()))
-  
+  }
 }
 
-#' Helper function to create an R6 class that inherits from the abstract `Dataset` class
-#' 
-#' All datasets that represent a map from keys to data samples should subclass this 
-#' class. All subclasses should overwrite the `.getitem()` method, which supports 
-#' fetching a data sample for a given key. Subclasses could also optionally 
-#' overwrite `.length()`, which is expected to return the size of the dataset 
-#' (e.g. number of samples) used by many sampler implementations 
+
+#' Helper function to create an function that generates R6 instances of class `dataset`
+#'
+#' All datasets that represent a map from keys to data samples should subclass this
+#' class. All subclasses should overwrite the `.getitem()` method, which supports
+#' fetching a data sample for a given key. Subclasses could also optionally
+#' overwrite `.length()`, which is expected to return the size of the dataset
+#' (e.g. number of samples) used by many sampler implementations
 #' and the default options of [dataloader()].
-#' 
+#'
+#' @returns
+#' The output is a function `f` with class `dataset_generator`. Calling `f()`
+#' creates a new instance of the R6 class `dataset`. The R6 class is stored in the
+#' enclosing environment of `f` and can also be accessed through `f`s attribute
+#' `Dataset`.
+#'
 #' @section Get a batch of observations:
-#' 
+#'
 #' By default datasets are iterated by returning each observation/item individually.
-#' Sometimes it's possible to have an optimized implementation to take a batch
+#' Often it's possible to have an optimized implementation to take a batch
 #' of observations (eg, subsetting a tensor by multiple indexes at once is faster than
 #' subsetting once for each index), in this case you can implement a `.getbatch` method
 #' that will be used instead of `.getitem` when getting a batch of observations within
-#' the dataloader.
+#' the dataloader. `.getbatch` must work for batches of size larger or equal to 1. For more
+#' on this see the the `vignette("loading-data")`.
 #' 
-#' @note 
+#' @note
 #' [dataloader()]  by default constructs a index
 #' sampler that yields integral indices.  To make it work with a map-style
 #' dataset with non-integral indices/keys, a custom sampler must be provided.
-#' 
+#'
 #' @param name a name for the dataset. It it's also used as the class
 #'   for it.
-#' @param inherit you can optionally inherit from a dataset when creating a 
+#' @param inherit you can optionally inherit from a dataset when creating a
 #'   new dataset.
 #' @param ... public methods for the dataset class
-#' @param parent_env An environment to use as the parent of newly-created 
+#' @param parent_env An environment to use as the parent of newly-created
 #'   objects.
 #' @inheritParams nn_module
-#' 
+#'
 #' @export
-dataset <- function(name = NULL, inherit = Dataset, ..., 
+dataset <- function(name = NULL, inherit = Dataset, ...,
                     private = NULL, active = NULL,
                     parent_env = parent.frame()) {
   create_class(
-    name = name, 
+    name = name,
     inherit = inherit,
     ...,
-    private = private, 
+    private = private,
     active = active,
     parent_env = parent_env,
     attr_name = "Dataset",
@@ -77,8 +84,9 @@ print.dataset_generator <- function(x, ...) {
 
 #' @export
 `[.dataset` <- function(x, y) {
-  if (length(y) > 1 && !is.null(x$.getbatch)) {
-    x$.getbatch(as.integer(y))
+  y <- as.integer(y)
+  if (!is.null(x$.getbatch)) {
+    x$.getbatch(y)
   } else {
     x$.getitem(y)
   }
@@ -90,9 +98,9 @@ length.dataset <- function(x) {
 }
 
 #' Dataset wrapping tensors.
-#' 
+#'
 #' Each sample will be retrieved by indexing tensors along the first dimension.
-#' 
+#'
 #' @param ... tensors that have the same size of the first dimension.
 #'
 #' @export
@@ -101,20 +109,20 @@ tensor_dataset <- dataset(
   initialize = function(...) {
     tensors <- rlang::list2(...)
     lens <- sapply(tensors, function(x) x$shape[1])
-    
-    if (!length(unique(lens)))
+
+    if (!length(unique(lens))) {
       value_error("all tensors must have the same size in the first dimension.")
-    
+    }
+
     self$tensors <- tensors
   },
   .getitem = function(index) {
-    
     if (is.list(index)) {
       index <- unlist(index)
     }
-    
+
     lapply(self$tensors, function(x) {
-        x[index, ..]
+      x[index, ..]
     })
   },
   .getbatch = function(index) {
@@ -135,21 +143,16 @@ tensor_dataset <- dataset(
 #' @export
 dataset_subset <- dataset(
   initialize = function(dataset, indices) {
-    self$dataset = dataset
-    self$indices = indices
+    self$dataset <- dataset
+    self$indices <- indices
+    if (!is.null(dataset$.getbatch)) {
+      self$.getbatch <- self$.getitem
+    }
   },
-  
   .getitem = function(idx) {
     return(self$dataset[self$indices[idx]])
   },
-  
   .length = function() {
     return(length(self$indices))
   }
 )
-
-
-
-
-
-
